@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2011-2017 Michael Goffioul
+Copyright (C) 2011-2019 Michael Goffioul
 
 This file is part of QConsole.
 
@@ -16,10 +16,25 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not,
-see <http://www.gnu.org/licenses/>.
+see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <algorithm>
+#include <csignal>
+#include <cstdio>
+#include <cstdarg>
+#include <cstring>
+
+#define WIN32_LEAN_AND_MEAN
+#if ! defined (_WIN32_WINNT) && ! defined (NTDDI_VERSION)
+#define _WIN32_WINNT 0x0500
+#endif
+#include <windows.h>
+#include <fcntl.h>
+#include <io.h>
+#include <versionhelpers.h>
+
 #include <QApplication>
 #include <QClipboard>
 #include <QColor>
@@ -40,20 +55,6 @@ see <http://www.gnu.org/licenses/>.
 #include <QUrl>
 #include <QMimeData>
 
-#include <fcntl.h>
-#include <io.h>
-#include <stdio.h>
-#include <stdarg.h>
-#define WIN32_LEAN_AND_MEAN
-#if ! defined (_WIN32_WINNT) && ! defined (NTDDI_VERSION)
-#define _WIN32_WINNT 0x0500
-#endif
-#include <windows.h>
-#include <versionhelpers.h>
-#include <cstring>
-#include <csignal>
-#include <limits>
-
 #include "QWinTerminalImpl.h"
 #include "QTerminalColors.h"
 
@@ -240,6 +241,7 @@ private:
   QSize m_charSize;
   QSize m_bufferSize;
   QRect m_consoleRect;
+  bool m_auto_scroll;
   QPoint m_cursorPos;
   bool m_cursorBlinking;
   bool m_hasBlinkingCursor;
@@ -273,18 +275,18 @@ static void maybeSwapPoints (QPoint& begin, QPoint& end)
 {
   if (end.y () < begin.y ()
       || (end.y () == begin.y () && end.x () < begin.x ()))
-    qSwap (begin, end);
+    std::swap (begin, end);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
 QConsolePrivate::QConsolePrivate (QWinTerminalImpl* parent, const QString& cmd)
-  : q (parent), m_command (cmd), m_cursorBlinking (false),
+  : q (parent), m_command (cmd), m_auto_scroll (true), m_cursorBlinking (false),
     m_hasBlinkingCursor (true), m_cursorType (BlockCursor),
     m_beginSelection (0, 0), m_endSelection (0, 0), m_settingSelection (false),
-    m_process (NULL), m_inWheelEvent (false)
+    m_process (nullptr), m_inWheelEvent (false)
 {
-  log (NULL);
+  log (nullptr);
 
   // Possibly detach from any existing console
   log ("Detaching from existing console (if any)...\n");
@@ -299,7 +301,7 @@ QConsolePrivate::QConsolePrivate (QWinTerminalImpl* parent, const QString& cmd)
 
   // Create new (hidden) console
   hOrigSta = GetProcessWindowStation ();
-  hNewSta = CreateWindowStation (NULL, 0, GENERIC_ALL, NULL);
+  hNewSta = CreateWindowStation (nullptr, 0, GENERIC_ALL, nullptr);
   log ("Current Windows station: %p.\nNew Windows station: %p.\n", hOrigSta,
        hNewSta);
   if (! SetProcessWindowStation (hNewSta))
@@ -328,9 +330,9 @@ QConsolePrivate::QConsolePrivate (QWinTerminalImpl* parent, const QString& cmd)
 
   log ("POSIX standard streams created.\n");
 
-  setvbuf (stdin, NULL, _IONBF, 0);
-  setvbuf (stdout, NULL, _IONBF, 0);
-  setvbuf (stderr, NULL, _IONBF, 0);
+  setvbuf (stdin, nullptr, _IONBF, 0);
+  setvbuf (stdout, nullptr, _IONBF, 0);
+  setvbuf (stderr, nullptr, _IONBF, 0);
 
   log ("POSIX standard stream buffers adjusted.\n");
 
@@ -371,7 +373,8 @@ QConsolePrivate::QConsolePrivate (QWinTerminalImpl* parent, const QString& cmd)
   m_font.setPointSize (9);
   m_font.setStyleHint (QFont::TypeWriter);
 
-  m_buffer = m_tmpBuffer = 0;
+  m_buffer = nullptr;
+  m_tmpBuffer = nullptr;
 
   m_consoleView = new QConsoleView (parent);
   m_horizontalScrollBar = new QScrollBar (Qt::Horizontal, parent);
@@ -390,13 +393,13 @@ QConsolePrivate::QConsolePrivate (QWinTerminalImpl* parent, const QString& cmd)
       SetConsoleOutputCP (65001);
     }
 
-  // Choose 15 (0xF) as index into the Windows console color map for the
-  // background and 0 (0x0) as the index for the foreground.  This
+  // Choose 0 (0x0) as index into the Windows console color map for the
+  // background and 7 (0x7) as the index for the foreground.  This
   // selection corresponds to the indices used in the foregroundColor,
   // setForegroundColor, backgroundColor, and SetBackgroundColor
   // functions.
 
-  SetConsoleTextAttribute (m_stdOut, 0xF0);
+  SetConsoleTextAttribute (m_stdOut, 0x07);
 
   // Defaults.
   setBackgroundColor (Qt::white);
@@ -408,8 +411,6 @@ QConsolePrivate::QConsolePrivate (QWinTerminalImpl* parent, const QString& cmd)
   QPalette palette (backgroundColor ());
   m_consoleView->setPalette (palette);
 
-  m_consoleView->setAutoFillBackground (true);
-
   m_consoleView->setFont (m_font);
   parent->setFocusPolicy (Qt::StrongFocus);
   parent->winId ();
@@ -456,10 +457,9 @@ QConsolePrivate::~QConsolePrivate (void)
       TerminateProcess (m_process, (UINT)-1);
       m_consoleThread->wait ();
     }
-  if (m_buffer)
-    delete [] m_buffer;
-  if (m_tmpBuffer)
-    delete [] m_tmpBuffer;
+
+  delete [] m_buffer;
+  delete [] m_tmpBuffer;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -543,6 +543,8 @@ QString QConsolePrivate::getSelection (void)
                 }
 
               QChar c (buf[i].Char.UnicodeChar);
+              if (c.isNull ())
+                c = QChar (' ');
 
               selection.append (c);
               if (! c.isSpace ())
@@ -579,12 +581,12 @@ void QConsolePrivate::clearSelection (void)
 
 QColor QConsolePrivate::backgroundColor (void) const
 {
-  return m_colors[15];
+  return m_colors[0];
 }
 
 QColor QConsolePrivate::foregroundColor (void) const
 {
-  return m_colors[0];
+  return m_colors[7];
 }
 
 QColor QConsolePrivate::selectionColor (void) const
@@ -599,15 +601,16 @@ QColor QConsolePrivate::cursorColor (void) const
 
 void QConsolePrivate::setBackgroundColor (const QColor& color)
 {
-  m_colors[15] = color;
+  m_colors[0] = color;
 
   QPalette palette (color);
+  palette.setColor(QPalette::Base, color);
   m_consoleView->setPalette (palette);
 }
 
 void QConsolePrivate::setForegroundColor (const QColor& color)
 {
-  m_colors[0] = color;
+  m_colors[7] = color;
 }
 
 void QConsolePrivate::setSelectionColor (const QColor& color)
@@ -638,7 +641,7 @@ void QConsolePrivate::drawTextBackground (QPainter& p, int cx1, int cy1,
 
   int ascent = p.fontMetrics ().ascent ();
   int stride = m_consoleRect.width ();
-  int y = ascent + cy1 * ch;;
+  int y = ascent + cy1 * ch;
 
   for (int j = cy1; j <= cy2; j++, y += ch)
     {
@@ -794,7 +797,7 @@ void QConsolePrivate::drawSelection (QPainter& p, int cx1, int cy1,
   int ascent = p.fontMetrics ().ascent ();
   int stride = m_consoleRect.width ();
 
-  int y = ascent + cy1 * ch;;
+  int y = ascent + cy1 * ch;
   for (int j = cy1; j <= cy2; j++, y += ch)
     {
       int charsThisLine = 0;
@@ -901,7 +904,7 @@ void QConsolePrivate::drawText (QPainter& p, int cx1, int cy1,
   int ascent = p.fontMetrics ().ascent ();
   int stride = m_consoleRect.width ();
 
-  int y = ascent + cy1 * ch;;
+  int y = ascent + cy1 * ch;
   for (int j = cy1; j <= cy2; j++, y += ch)
     {
       // Reset string buffer and starting X coordinate
@@ -1058,29 +1061,14 @@ void QConsolePrivate::syncConsoleParameters (void)
   COORD bs;
   SMALL_RECT sr;
 
-  bs.X = sbi.dwSize.X;
+  bs.X = m_bufferSize.width ();
   bs.Y = m_bufferSize.height ();
-  sr.Left   = sbi.srWindow.Left;
-  sr.Right  = sbi.srWindow.Right;
-  sr.Top    = m_consoleRect.top ();
+  sr.Left = m_consoleRect.left ();
+  sr.Right = m_consoleRect.right ();
+  sr.Top = m_consoleRect.top ();
   sr.Bottom = m_consoleRect.bottom ();
 
-  if (bs.Y > sbi.dwSize.Y)
-    {
-      SetConsoleScreenBufferSize (hStdOut, bs);
-      SetConsoleWindowInfo (hStdOut, TRUE, &sr);
-    }
-  else
-    {
-      SetConsoleWindowInfo (hStdOut, TRUE, &sr);
-      SetConsoleScreenBufferSize (hStdOut, bs);
-    }
-
-  bs.X = m_bufferSize.width ();
-  sr.Left  = m_consoleRect.left ();
-  sr.Right = m_consoleRect.right ();
-
-  if (bs.X > sbi.dwSize.X)
+  if (bs.X * bs.Y > sbi.dwSize.X * sbi.dwSize.Y)
     {
       SetConsoleScreenBufferSize (hStdOut, bs);
       SetConsoleWindowInfo (hStdOut, TRUE, &sr);
@@ -1096,12 +1084,10 @@ void QConsolePrivate::syncConsoleParameters (void)
   log ("  window: (%d, %d) -> (%d, %d)\n",
        sr.Left, sr.Top, sr.Right, sr.Bottom);
 
-  if (m_buffer)
-    delete [] m_buffer;
-  if (m_tmpBuffer)
-    delete [] m_tmpBuffer;
+  delete [] m_buffer;
+  delete [] m_tmpBuffer;
 
-  int bufSize = m_consoleRect.width () * m_consoleRect.height ();
+  std::size_t bufSize = m_consoleRect.width () * m_consoleRect.height ();
 
   m_buffer = new CHAR_INFO[bufSize];
   m_tmpBuffer = new CHAR_INFO[bufSize];
@@ -1124,6 +1110,7 @@ void QConsolePrivate::grabConsoleBuffer (CHAR_INFO* buf)
   r.Right  = m_consoleRect.right ();
   r.Bottom = m_consoleRect.bottom ();
 
+  log ("ReadConsoleOutput (%d,%d) -> (%d,%d)\n", r.Left, r.Top, r.Right, r.Bottom);
   if (! ReadConsoleOutput (m_stdOut, (buf ? buf : m_buffer), bs, bc, &r))
     qCritical ("cannot read console output");
 }
@@ -1216,6 +1203,16 @@ void QConsolePrivate::setVerticalScrollValue (int value)
   if (SetConsoleWindowInfo (hStdOut, TRUE, &r))
     {
       m_consoleRect.moveTop (value);
+
+      CONSOLE_SCREEN_BUFFER_INFO sbi;
+      if (GetConsoleScreenBufferInfo (hStdOut, &sbi))
+        {
+          if (sbi.dwCursorPosition.Y > m_consoleRect.bottom ())
+            m_auto_scroll = false;
+          else
+            m_auto_scroll = true;
+        }
+
       updateConsoleView ();
     }
 }
@@ -1250,6 +1247,8 @@ void QConsolePrivate::monitorConsole (void)
 
   if (GetConsoleScreenBufferInfo (hStdOut, &sbi))
     {
+      bool need_console_sync = false;
+
       if (m_bufferSize.width () != sbi.dwSize.X
           || m_bufferSize.height () != sbi.dwSize.Y)
         {
@@ -1258,6 +1257,7 @@ void QConsolePrivate::monitorConsole (void)
           m_bufferSize.rheight () = sbi.dwSize.Y;
           updateHorizontalScrollBar ();
           updateVerticalScrollBar ();
+          need_console_sync = true;
         }
 
       if (m_cursorPos.x () != sbi.dwCursorPosition.X
@@ -1278,29 +1278,39 @@ void QConsolePrivate::monitorConsole (void)
 
       if (m_consoleRect.left () != sbi.srWindow.Left
           || m_consoleRect.right () != sbi.srWindow.Right
-          || m_consoleRect.top () != sbi.srWindow.Top
-          || m_consoleRect.bottom () != sbi.srWindow.Bottom)
+          || (m_auto_scroll &&
+              (m_consoleRect.top () != sbi.srWindow.Top
+               || m_consoleRect.bottom () != sbi.srWindow.Bottom)))
         {
           // Console window changed
+          log ("--> Console window changed\n");
           m_consoleRect = QRect (sbi.srWindow.Left, sbi.srWindow.Top,
                                  sbi.srWindow.Right - sbi.srWindow.Left + 1,
                                  sbi.srWindow.Bottom - sbi.srWindow.Top + 1);
           updateHorizontalScrollBar ();
           updateVerticalScrollBar ();
+          syncConsoleParameters ();
           updateConsoleView ();
           return;
         }
 
+      if (need_console_sync)
+      {
+        syncConsoleParameters ();
+        updateConsoleView ();
+        return;
+      }
+
       if (m_tmpBuffer && m_buffer)
         {
           grabConsoleBuffer (m_tmpBuffer);
           if (memcmp (m_tmpBuffer, m_buffer,
-                      sizeof (CHAR_INFO) * m_consoleRect.width () *
-                      m_consoleRect.height ()))
+                      sizeof (CHAR_INFO) * m_consoleRect.width ()
+                      * m_consoleRect.height ()))
             {
               // FIXME: compute the area to update based on the
               // difference between the 2 buffers.
-              qSwap (m_buffer, m_tmpBuffer);
+              std::swap (m_buffer, m_tmpBuffer);
               updateConsoleView (false);
             }
         }
@@ -1318,21 +1328,21 @@ void QConsolePrivate::startCommand (void)
 
   if (! cmd.isEmpty ())
     {
-      STARTUPINFO si;
+      STARTUPINFOW si;
       PROCESS_INFORMATION pi;
 
       ZeroMemory (&si, sizeof (si));
       si.cb = sizeof (si);
       ZeroMemory (&pi, sizeof (pi));
 
-      if (CreateProcessW (NULL,
+      if (CreateProcessW (nullptr,
                           (LPWSTR)cmd.unicode (),
-                          NULL,
-                          NULL,
+                          nullptr,
+                          nullptr,
                           TRUE,
                           0,
-                          NULL,
-                          NULL,
+                          nullptr,
+                          nullptr,
                           &si,
                           &pi))
         {
@@ -1340,7 +1350,7 @@ void QConsolePrivate::startCommand (void)
           m_process = pi.hProcess;
           WaitForSingleObject (m_process, INFINITE);
           CloseHandle (m_process);
-          m_process = NULL;
+          m_process = nullptr;
         }
     }
 }
@@ -1356,6 +1366,8 @@ void QConsolePrivate::sendConsoleText (const QString& s)
 
   // clear any selection on inserting text
   clearSelection();
+  // enable auto-scrolling
+  m_auto_scroll = true;
 
   int len = s.length ();
   INPUT_RECORD events[TEXT_CHUNK_SIZE];
@@ -1425,32 +1437,18 @@ QConsolePrivate::cursorRect (void)
 
 //////////////////////////////////////////////////////////////////////////////
 
-QWinTerminalImpl::QWinTerminalImpl (QWidget* parent)
-    : QTerminal (parent), d (new QConsolePrivate (this)),
+QWinTerminalImpl::QWinTerminalImpl (octave::base_qobject& oct_qobj,
+                                    QWidget* parent)
+    : QTerminal (oct_qobj, parent), d (new QConsolePrivate (this)),
       allowTripleClick (false)
 {
     installEventFilter (this);
 
-    connect (this, SIGNAL (set_global_shortcuts_signal (bool)),
-           parent, SLOT (set_global_shortcuts (bool)));
-    connect (this, SIGNAL (set_global_shortcuts_signal (bool)),
-             this, SLOT (set_global_shortcuts (bool)));
-
-    connect (this, SIGNAL (set_screen_size_signal (int, int)),
-             parent, SLOT (set_screen_size (int, int)));
-
     setAcceptDrops (true);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-QWinTerminalImpl::QWinTerminalImpl (const QString& cmd, QWidget* parent)
-    : QTerminal (parent), d (new QConsolePrivate (this, cmd))
-{
-}
-
-//////////////////////////////////////////////////////////////////////////////
-
 QWinTerminalImpl::~QWinTerminalImpl (void)
 {
   delete d;
@@ -1540,6 +1538,7 @@ void QWinTerminalImpl::viewPaintEvent (QConsoleView* w, QPaintEvent* event)
   int ch = d->m_charSize.height ();
 
   QRect updateRect = event->rect ();
+  p.fillRect(updateRect, QBrush(d->backgroundColor()));
 
   int cx1 = updateRect.left () / cw;
   int cy1 = updateRect.top () / ch;
@@ -1635,8 +1634,6 @@ void QWinTerminalImpl::updateSelection (void)
 
 void QWinTerminalImpl::focusInEvent (QFocusEvent* event)
 {
-  emit set_global_shortcuts_signal (false);   // disable some shortcuts
-
   setBlinkingCursorState (true);
 
   QWidget::focusInEvent (event);
@@ -1644,8 +1641,6 @@ void QWinTerminalImpl::focusInEvent (QFocusEvent* event)
 
 void QWinTerminalImpl::focusOutEvent (QFocusEvent* event)
 {
-  emit set_global_shortcuts_signal (true);    // re-enable shortcuts
-
   // Force the cursor to be redrawn.
   d->m_cursorBlinking = true;
 
@@ -1742,7 +1737,7 @@ void QWinTerminalImpl::setCursorColor (bool useForegroundColor,
   d->setCursorColor (useForegroundColor, color);
 }
 
-void QWinTerminalImpl::setScrollBufferSize(int value)
+void QWinTerminalImpl::setScrollBufferSize (int value)
 {
   d->setScrollBufferSize (value);
 }
@@ -1797,7 +1792,15 @@ void QWinTerminalImpl::pasteClipboard (void)
   QString text = QApplication::clipboard()->text (QClipboard::Clipboard);
 
   if (! text.isEmpty ())
+    {
+      if (text.contains ("\t"))
+        {
+          qWarning ("Tabs replaced with spaces in pasted text before processing");
+          text.replace ("\t", "    ");
+        }
+
     sendText (text);
+    }
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1840,7 +1843,7 @@ void QWinTerminalImpl::dropEvent (QDropEvent *event)
       foreach (QUrl url, event->mimeData ()->urls ())
         {
           if(dropText.length () > 0)
-            dropText += "\n";
+            dropText += '\n';
           dropText  += url.toLocalFile ();
         }
       sendText (dropText);
