@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2020 The Octave Project Developers
+// Copyright (C) 1996-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -50,6 +50,7 @@
 #include "oct-time.h"
 #include "quit.h"
 #include "str-vec.h"
+#include "unistr-wrappers.h"
 
 #include "Cell.h"
 #include "defaults.h"
@@ -511,8 +512,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
   if (read_mat5_tag (is, swap, type, element_length, is_small_data_element))
     return retval;                      // EOF
 
-  octave::interpreter& interp
-    = octave::__get_interpreter__ ("read_mat5_binary_element");
+  octave::interpreter& interp = octave::__get_interpreter__ ();
 
   if (type == miCOMPRESSED)
     {
@@ -528,9 +528,10 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
       // We uncompress the first 8 bytes of the header to get the buffer length
       // This will fail with an error Z_MEM_ERROR
       uLongf destLen = 8;
+      uLongf elt_len = element_length;
       OCTAVE_LOCAL_BUFFER (unsigned int, tmp, 2);
-      if (uncompress (reinterpret_cast<Bytef *> (tmp), &destLen,
-                      reinterpret_cast<Bytef *> (inbuf), element_length)
+      if (uncompress2 (reinterpret_cast<Bytef *> (tmp), &destLen,
+                       reinterpret_cast<Bytef *> (inbuf), &elt_len)
           == Z_MEM_ERROR)
         error ("load: error probing size of compressed data element");
 
@@ -543,10 +544,19 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
       // FIXME: find a way to avoid casting away const here!
 
-      int err = uncompress (reinterpret_cast<Bytef *>
-                            (const_cast<char *> (outbuf.c_str ())),
-                            &destLen, reinterpret_cast<Bytef *> (inbuf),
-                            element_length);
+      elt_len = element_length;
+      int err = uncompress2 (reinterpret_cast<Bytef *>
+                             (const_cast<char *> (outbuf.c_str ())),
+                             &destLen, reinterpret_cast<Bytef *> (inbuf),
+                             &elt_len);
+
+      // Ignore buffer error if we have consumed all the input buffer
+      // and uncompressing the data generated as many bytes of output as
+      // we were expecting given the data element size that was stored
+      // in the Matlab data element header.
+      if (err == Z_BUF_ERROR && destLen == tmp[1] + 8
+          && elt_len == static_cast<uLongf> (element_length))
+        err = Z_OK;
 
       if (err != Z_OK)
         {
@@ -891,7 +901,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
                     if (fs.exists ())
                       {
-                        size_t xpos
+                        std::size_t xpos
                           = str.find_last_of (octave::sys::file_ops::dir_sep_chars ());
 
                         std::string dir_name = str.substr (0, xpos);
@@ -902,7 +912,8 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
                         if (ov_fcn.is_defined ())
                           // XXX FCN_HANDLE: SIMPLE/SCOPED
-                          tc = octave_value (new octave_fcn_handle (ov_fcn, fname));
+                          tc = octave_value (new octave_fcn_handle (ov_fcn,
+                                             fname));
                       }
                     else
                       {
@@ -918,7 +929,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
                         str = octave::sys::env::make_absolute (p.find_first_of (names));
 
-                        size_t xpos
+                        std::size_t xpos
                           = str.find_last_of (octave::sys::file_ops::dir_sep_chars ());
 
                         std::string dir_name = str.substr (0, xpos);
@@ -929,7 +940,8 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
                         if (ov_fcn.is_defined ())
                           // XXX FCN_HANDLE: SIMPLE/SCOPED
-                          tc = octave_value (new octave_fcn_handle (ov_fcn, fname));
+                          tc = octave_value (new octave_fcn_handle (ov_fcn,
+                                             fname));
                         else
                           {
                             warning_with_id ("Octave:load:file-not-found",
@@ -941,7 +953,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
                   }
                 else
                   {
-                    size_t xpos
+                    std::size_t xpos
                       = fpath.find_last_of (octave::sys::file_ops::dir_sep_chars ());
 
                     std::string dir_name = fpath.substr (0, xpos);
@@ -979,20 +991,24 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
             m2 = subsys_ov.scalar_map_value ();
             m2 = m2.contents ("MCOS").scalar_map_value ();
             tc2 = m2.contents ("MCOS").cell_value ()(1 + off).cell_value ()(1);
-            m2 = tc2.scalar_map_value ();
 
             octave::stack_frame::local_vars_map local_vars;
 
-            if (m2.nfields () > 0)
+            if (! tc2.isempty ())
               {
-                octave_value tmp;
+                m2 = tc2.scalar_map_value ();
 
-                for (auto p0 = m2.begin (); p0 != m2.end (); p0++)
+                if (m2.nfields () > 0)
                   {
-                    std::string key = m2.key (p0);
-                    octave_value val = m2.contents (p0);
+                    octave_value tmp;
 
-                    local_vars[key] = val;
+                    for (auto p0 = m2.begin (); p0 != m2.end (); p0++)
+                      {
+                        std::string key = m2.key (p0);
+                        octave_value val = m2.contents (p0);
+
+                        local_vars[key] = val;
+                      }
                   }
               }
 
@@ -1026,7 +1042,8 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
               error ("load: failed to load anonymous function handle");
 
             // XXX FCN_HANDLE: ANONYMOUS
-            tc = octave_value (new octave_fcn_handle (fh->fcn_val (), local_vars));
+            tc = octave_value (new octave_fcn_handle (fh->fcn_val (),
+                               local_vars));
           }
         else
           error ("load: invalid function handle type");
@@ -1124,7 +1141,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
           classname = name;
         }
       }
-      // Fall-through
+    // Fall-through
 
     case MAT_FILE_STRUCT_CLASS:
       {
@@ -1349,7 +1366,7 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
       break;
 
     case MAT_FILE_CHAR_CLASS:
-      // handle as a numerical array to start with
+    // handle as a numerical array to start with
 
     case MAT_FILE_DOUBLE_CLASS:
     default:
@@ -1413,58 +1430,84 @@ read_mat5_binary_element (std::istream& is, const std::string& filename,
 
             tc = ctmp;
           }
-        else
+        else if (arrayclass == MAT_FILE_CHAR_CLASS)
           {
-            if (arrayclass == MAT_FILE_CHAR_CLASS)
+            bool converted = false;
+            if (re.isvector () && (type == miUTF16 || type == miUINT16))
               {
-                if (type == miUTF16 || type == miUTF32)
-                  {
-                    bool found_big_char = false;
-                    for (octave_idx_type i = 0; i < n; i++)
-                      {
-                        if (re(i) > 127)
-                          {
-                            re(i) = '?';
-                            found_big_char = true;
-                          }
-                      }
+                uint16NDArray u16 = re;
+                const uint16_t *u16_str
+                  = reinterpret_cast<const uint16_t *> (u16.data ());
 
-                    if (found_big_char)
-                      warning_with_id ("Octave:load:unsupported-utf-char",
-                               "load: can not read non-ASCII portions of UTF characters; replacing unreadable characters with '?'");
+                // Convert to UTF-8.
+                std::size_t n8;
+                uint8_t *u8_str = octave_u16_to_u8_wrapper (u16_str,
+                                  u16.numel (),
+                                  nullptr, &n8);
+                if (u8_str)
+                  {
+                    // FIXME: Is there a better way to construct a charMatrix
+                    // from a non zero terminated buffer?
+                    tc = charMatrix (std::string (reinterpret_cast<char *> (u8_str), n8));
+                    free (u8_str);
+                    converted = true;
                   }
-                else if (type == miUTF8)
+              }
+            else if (re.isvector () && (type == miUTF32 || type == miUINT32))
+              {
+                uint32NDArray u32 = re;
+                const uint32_t *u32_str
+                  = reinterpret_cast<const uint32_t *> (u32.data ());
+
+                // Convert to UTF-8.
+                std::size_t n8;
+                uint8_t *u8_str = octave_u32_to_u8_wrapper (u32_str,
+                                  u32.numel (),
+                                  nullptr, &n8);
+                if (u8_str)
                   {
-                    // Search for multi-byte encoded UTF8 characters and
-                    // replace with 0x3F for '?'...  Give the user a warning
+                    // FIXME: Is there a better way to construct a charMatrix
+                    // from a non zero terminated buffer?
+                    tc = charMatrix (std::string (reinterpret_cast<char *> (u8_str), n8));
+                    free (u8_str);
+                    converted = true;
+                  }
+              }
+            else if (type == miUTF8 || type == miUINT8)
+              {
+                // Octave's internal encoding is UTF-8.  So we should be
+                // able to use this natively.
+                tc = re;
+                tc = tc.convert_to_str (false, true, '\'');
+                converted = true;
+              }
 
-                    bool utf8_multi_byte = false;
-                    for (octave_idx_type i = 0; i < n; i++)
+            if (! converted)
+              {
+                // Fall back to manually replacing non-ASCII
+                // characters by "?".
+                bool found_big_char = false;
+                for (octave_idx_type i = 0; i < n; i++)
+                  {
+                    if (re(i) > 127)
                       {
-                        unsigned char a = static_cast<unsigned char> (re(i));
-                        if (a > 0x7f)
-                          utf8_multi_byte = true;
-                      }
-
-                    if (utf8_multi_byte)
-                      {
-                        warning_with_id ("Octave:load:unsupported-utf-char",
-                                         "load: can not read multi-byte encoded UTF8 characters; replacing unreadable characters with '?'");
-                        for (octave_idx_type i = 0; i < n; i++)
-                          {
-                            unsigned char a
-                              = static_cast<unsigned char> (re(i));
-                            if (a > 0x7f)
-                              re(i) = '?';
-                          }
+                        re(i) = '?';
+                        found_big_char = true;
                       }
                   }
+
+                if (found_big_char)
+                  warning_with_id ("Octave:load:unsupported-utf-char",
+                                   "load: failed to convert from input to UTF-8; "
+                                   "replacing non-ASCII characters with '?'");
+
                 tc = re;
                 tc = tc.convert_to_str (false, true, '\'');
               }
-            else
-              tc = re;
+
           }
+        else
+          tc = re;
       }
     }
 
@@ -1524,7 +1567,8 @@ read_mat5_binary_file_header (std::istream& is, bool& swap, bool quiet,
   if (swap)
     swap_bytes<8> (&subsys_offset, 1);
 
-  if (subsys_offset != 0x2020202020202020ULL && subsys_offset != 0ULL)
+  if (subsys_offset != UINT64_C (0x2020202020202020)
+      && subsys_offset != UINT64_C (0))
     {
       // Read the subsystem data block
       is.seekg (subsys_offset, std::ios::beg);
@@ -1622,7 +1666,7 @@ write_mat5_array (std::ostream& os, const NDArray& m, bool save_as_floats)
 
   double max_val, min_val;
   if (m.all_integers (max_val, min_val))
-    st = get_save_type (max_val, min_val);
+    st = octave::get_save_type (max_val, min_val);
 
   mat5_data_type mst;
   int size;
@@ -1707,7 +1751,7 @@ write_mat5_array (std::ostream& os, const FloatNDArray& m, bool)
 
   float max_val, min_val;
   if (m.all_integers (max_val, min_val))
-    st = get_save_type (max_val, min_val);
+    st = octave::get_save_type (max_val, min_val);
 
   mat5_data_type mst;
   int size;
@@ -1921,9 +1965,9 @@ save_mat5_array_length (const double *val, octave_idx_type nel,
             size = 4;
         }
 
-      // The code below is disabled since get_save_type currently doesn't
-      // deal with integer types.  This will need to be activated if
-      // get_save_type is changed.
+      // The code below is disabled since get_save_type currently
+      // doesn't deal with integer types.  This will need to be
+      // activated if get_save_type is changed.
 
       // double max_val = val[0];
       // double min_val = val[0];
@@ -1969,15 +2013,15 @@ save_mat5_array_length (const double *val, octave_idx_type nel,
 }
 
 int
-save_mat5_array_length (const float* /* val */, octave_idx_type nel, bool)
+save_mat5_array_length (const float * /* val */, octave_idx_type nel, bool)
 {
   if (nel > 0)
     {
       int size = 4;
 
-      // The code below is disabled since get_save_type currently doesn't
-      // deal with integer types.  This will need to be activated if
-      // get_save_type is changed.
+      // The code below is disabled since get_save_type currently
+      // doesn't deal with integer types.  This will need to be
+      // activated if get_save_type is changed.
 
       // float max_val = val[0];
       // float min_val = val[0];
@@ -2066,12 +2110,30 @@ save_mat5_array_length (const FloatComplex *val, octave_idx_type nel,
   return ret;
 }
 
+static uint16_t *
+maybe_convert_to_u16 (const charNDArray& chm, std::size_t& n16_str)
+{
+  uint16_t *u16_str;
+  dim_vector dv = chm.dims ();
+
+  if (chm.ndims () == 2 && dv(0) == 1)
+    {
+      const uint8_t *u8_str = reinterpret_cast<const uint8_t *> (chm.data ());
+      u16_str = octave_u8_to_u16_wrapper (u8_str, chm.numel (),
+                                          nullptr, &n16_str);
+    }
+  else
+    u16_str = nullptr;
+
+  return u16_str;
+}
+
 int
 save_mat5_element_length (const octave_value& tc, const std::string& name,
                           bool save_as_floats, bool mat7_format)
 {
-  size_t max_namelen = 63;
-  size_t len = name.length ();
+  std::size_t max_namelen = 63;
+  std::size_t len = name.length ();
   std::string cname = tc.class_name ();
   int ret = 32;
 
@@ -2083,9 +2145,25 @@ save_mat5_element_length (const octave_value& tc, const std::string& name,
   if (tc.is_string ())
     {
       charNDArray chm = tc.char_array_value ();
+      // convert to UTF-16
+      std::size_t n16_str;
+      uint16_t *u16_str = maybe_convert_to_u16 (chm, n16_str);
       ret += 8;
-      if (chm.numel () > 2)
-        ret += PAD (2 * chm.numel ());
+
+      octave_idx_type str_len;
+      std::size_t sz_of = 1;
+      if (u16_str)
+        {
+          free (u16_str);
+          // Count number of elements in converted string
+          str_len = n16_str;
+          sz_of = 2;
+        }
+      else
+        str_len = chm.numel ();
+
+      if (str_len > 2)
+        ret += PAD (sz_of * str_len);
     }
   else if (tc.issparse ())
     {
@@ -2125,95 +2203,90 @@ save_mat5_element_length (const octave_value& tc, const std::string& name,
 
   else if (cname == "int8")
     INT_LEN (tc.int8_array_value ().numel (), 1)
-  else if (cname == "int16")
-    INT_LEN (tc.int16_array_value ().numel (), 2)
-  else if (cname == "int32")
-    INT_LEN (tc.int32_array_value ().numel (), 4)
-  else if (cname == "int64")
-    INT_LEN (tc.int64_array_value ().numel (), 8)
-  else if (cname == "uint8")
-    INT_LEN (tc.uint8_array_value ().numel (), 1)
-  else if (cname == "uint16")
-    INT_LEN (tc.uint16_array_value ().numel (), 2)
-  else if (cname == "uint32")
-    INT_LEN (tc.uint32_array_value ().numel (), 4)
-  else if (cname == "uint64")
-    INT_LEN (tc.uint64_array_value ().numel (), 8)
-  else if (tc.islogical ())
-    INT_LEN (tc.bool_array_value ().numel (), 1)
-  else if (tc.is_real_scalar () || tc.is_real_matrix () || tc.is_range ())
-    {
-      if (tc.is_single_type ())
-        {
-          const FloatNDArray m = tc.float_array_value ();
-          ret += save_mat5_array_length (m.fortran_vec (), m.numel (),
-                                         save_as_floats);
-        }
-      else
-        {
-          const NDArray m = tc.array_value ();
-          ret += save_mat5_array_length (m.fortran_vec (), m.numel (),
-                                         save_as_floats);
-        }
-    }
-  else if (tc.iscell ())
-    {
-      Cell cell = tc.cell_value ();
-      octave_idx_type nel = cell.numel ();
+    else if (cname == "int16")
+      INT_LEN (tc.int16_array_value ().numel (), 2)
+      else if (cname == "int32")
+        INT_LEN (tc.int32_array_value ().numel (), 4)
+        else if (cname == "int64")
+          INT_LEN (tc.int64_array_value ().numel (), 8)
+          else if (cname == "uint8")
+            INT_LEN (tc.uint8_array_value ().numel (), 1)
+            else if (cname == "uint16")
+              INT_LEN (tc.uint16_array_value ().numel (), 2)
+              else if (cname == "uint32")
+                INT_LEN (tc.uint32_array_value ().numel (), 4)
+                else if (cname == "uint64")
+                  INT_LEN (tc.uint64_array_value ().numel (), 8)
+                  else if (tc.islogical ())
+                    INT_LEN (tc.bool_array_value ().numel (), 1)
+                    else if (tc.is_real_scalar () || tc.is_real_matrix () || tc.is_range ())
+                      {
+                        if (tc.is_single_type ())
+                          {
+                            const FloatNDArray m = tc.float_array_value ();
+                            ret += save_mat5_array_length (m.data (), m.numel (), save_as_floats);
+                          }
+                        else
+                          {
+                            const NDArray m = tc.array_value ();
+                            ret += save_mat5_array_length (m.data (), m.numel (), save_as_floats);
+                          }
+                      }
+                    else if (tc.iscell ())
+                      {
+                        Cell cell = tc.cell_value ();
+                        octave_idx_type nel = cell.numel ();
 
-      for (int i = 0; i < nel; i++)
-        ret += 8 +
-          save_mat5_element_length (cell (i), "", save_as_floats, mat7_format);
-    }
-  else if (tc.is_complex_scalar () || tc.is_complex_matrix ())
-    {
-      if (tc.is_single_type ())
-        {
-          const FloatComplexNDArray m = tc.float_complex_array_value ();
-          ret += save_mat5_array_length (m.fortran_vec (), m.numel (),
-                                         save_as_floats);
-        }
-      else
-        {
-          const ComplexNDArray m = tc.complex_array_value ();
-          ret += save_mat5_array_length (m.fortran_vec (), m.numel (),
-                                         save_as_floats);
-        }
-    }
-  else if (tc.isstruct () || tc.is_inline_function () || tc.isobject ())
-    {
-      int fieldcnt = 0;
-      const octave_map m = tc.map_value ();
-      octave_idx_type nel = m.numel ();
+                        for (int i = 0; i < nel; i++)
+                          ret += 8 +
+                                 save_mat5_element_length (cell (i), "", save_as_floats, mat7_format);
+                      }
+                    else if (tc.is_complex_scalar () || tc.is_complex_matrix ())
+                      {
+                        if (tc.is_single_type ())
+                          {
+                            const FloatComplexNDArray m = tc.float_complex_array_value ();
+                            ret += save_mat5_array_length (m.data (), m.numel (), save_as_floats);
+                          }
+                        else
+                          {
+                            const ComplexNDArray m = tc.complex_array_value ();
+                            ret += save_mat5_array_length (m.data (), m.numel (), save_as_floats);
+                          }
+                      }
+                    else if (tc.isstruct () || tc.is_inline_function () || tc.isobject ())
+                      {
+                        int fieldcnt = 0;
+                        const octave_map m = tc.map_value ();
+                        octave_idx_type nel = m.numel ();
 
-      if (tc.is_inline_function ())
-        ret += 8 + PAD (6);  // length of "inline" is 6
-      else if (tc.isobject ())
-        {
-          size_t classlen = tc.class_name ().length ();
+                        if (tc.is_inline_function ())
+                          ret += 8 + PAD (6);  // length of "inline" is 6
+                        else if (tc.isobject ())
+                          {
+                            std::size_t classlen = tc.class_name ().length ();
 
-          ret += 8 + PAD (classlen > max_namelen ? max_namelen : classlen);
-        }
+                            ret += 8 + PAD (classlen > max_namelen ? max_namelen : classlen);
+                          }
 
-      for (auto i = m.begin (); i != m.end (); i++)
-        fieldcnt++;
+                        for (auto i = m.begin (); i != m.end (); i++)
+                          fieldcnt++;
 
-      ret += 16 + fieldcnt * (max_namelen + 1);
+                        ret += 16 + fieldcnt * (max_namelen + 1);
 
-      for (octave_idx_type j = 0; j < nel; j++)
-        {
+                        for (octave_idx_type j = 0; j < nel; j++)
+                          {
+                            for (auto i = m.begin (); i != m.end (); i++)
+                              {
+                                const Cell elts = m.contents (i);
 
-          for (auto i = m.begin (); i != m.end (); i++)
-            {
-              const Cell elts = m.contents (i);
-
-              ret += 8 + save_mat5_element_length (elts(j), "", save_as_floats,
-                                                   mat7_format);
-            }
-        }
-    }
-  else
-    ret = -1;
+                                ret += 8 + save_mat5_element_length (elts(j), "", save_as_floats,
+                                                                     mat7_format);
+                              }
+                          }
+                      }
+                    else
+                      ret = -1;
 
   return ret;
 }
@@ -2253,7 +2326,7 @@ save_mat5_binary_element (std::ostream& os,
   int32_t flags = 0;
   int32_t nnz_32 = 0;
   std::string cname = tc.class_name ();
-  size_t max_namelen = 63;
+  std::size_t max_namelen = 63;
 
   dim_vector dv = tc.dims ();
   int nd = tc.ndims ();
@@ -2321,12 +2394,12 @@ save_mat5_binary_element (std::ostream& os,
           // + 12 bytes.  Reality is it must be larger again than that.
           std::string buf_str = buf.str ();
           uLongf srcLen = buf_str.length ();
-          uLongf destLen = srcLen * 101 / 100 + 12;
+          uLongf destLen = compressBound (srcLen);
           OCTAVE_LOCAL_BUFFER (char, out_buf, destLen);
 
           if (compress (reinterpret_cast<Bytef *> (out_buf), &destLen,
                         reinterpret_cast<const Bytef *> (buf_str.c_str ()),
-                                                         srcLen)
+                        srcLen)
               != Z_OK)
             error ("save: error compressing data element");
 
@@ -2410,21 +2483,42 @@ save_mat5_binary_element (std::ostream& os,
 
   write_mat5_tag (os, miINT32, dim_len);
 
-  for (int i = 0; i < nd; i++)
+  // Strings need to be converted here (or dim-vector will be off).
+  charNDArray chm;
+  uint16_t *u16_str;
+  std::size_t n16_str;
+  bool conv_u16 = false;
+  if (tc.is_string ())
     {
-      int32_t n = dv(i);
-      os.write (reinterpret_cast<char *> (&n), 4);
+      chm = tc.char_array_value ();
+      u16_str = maybe_convert_to_u16 (chm, n16_str);
+
+      if (u16_str)
+        conv_u16 = true;
     }
 
+  if (conv_u16)
+    {
+      int32_t n = 1;
+      os.write (reinterpret_cast<char *> (&n), 4);
+      os.write (reinterpret_cast<char *> (&n16_str), 4);
+    }
+  else
+    for (int i = 0; i < nd; i++)
+      {
+        int32_t n = dv(i);
+        os.write (reinterpret_cast<char *> (&n), 4);
+      }
+
   if (PAD (dim_len) > dim_len)
     {
-      static char buf[9]="\x00\x00\x00\x00\x00\x00\x00\x00";
+      static char buf[9] = "\x00\x00\x00\x00\x00\x00\x00\x00";
       os.write (buf, PAD (dim_len) - dim_len);
     }
 
   // array name subelement
   {
-    size_t namelen = name.length ();
+    std::size_t namelen = name.length ();
 
     if (namelen > max_namelen)
       namelen = max_namelen;  // Truncate names if necessary
@@ -2441,24 +2535,35 @@ save_mat5_binary_element (std::ostream& os,
   // data element
   if (tc.is_string ())
     {
-      charNDArray chm = tc.char_array_value ();
-      octave_idx_type nel = chm.numel ();
-      octave_idx_type len = nel*2;
-      octave_idx_type paddedlength = PAD (len);
+      octave_idx_type len;
+      octave_idx_type paddedlength;
 
-      OCTAVE_LOCAL_BUFFER (int16_t, buf, nel+3);
-      write_mat5_tag (os, miUINT16, len);
+      if (conv_u16)
+        {
+          // converted UTF-16
+          len = n16_str*2;
+          paddedlength = PAD (len);
 
-      const char *s = chm.data ();
+          write_mat5_tag (os, miUTF16, len);
 
-      for (octave_idx_type i = 0; i < nel; i++)
-        buf[i] = *s++ & 0x00FF;
+          os.write (reinterpret_cast<char *> (u16_str), len);
 
-      os.write (reinterpret_cast<char *> (buf), len);
+          free (u16_str);
+        }
+      else
+        {
+          // write as UTF-8
+          len = chm.numel ();
+          paddedlength = PAD (len);
+
+          write_mat5_tag (os, miUTF8, len);
+
+          os.write (chm.data (), len);
+        }
 
       if (paddedlength > len)
         {
-          static char padbuf[9]="\x00\x00\x00\x00\x00\x00\x00\x00";
+          static char padbuf[9] = "\x00\x00\x00\x00\x00\x00\x00\x00";
           os.write (padbuf, paddedlength - len);
         }
     }
@@ -2508,55 +2613,55 @@ save_mat5_binary_element (std::ostream& os,
     {
       int8NDArray m = tc.int8_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), -1, m.numel ());
+      write_mat5_integer_data (os, m.data (), -1, m.numel ());
     }
   else if (cname == "int16")
     {
       int16NDArray m = tc.int16_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), -2, m.numel ());
+      write_mat5_integer_data (os, m.data (), -2, m.numel ());
     }
   else if (cname == "int32")
     {
       int32NDArray m = tc.int32_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), -4, m.numel ());
+      write_mat5_integer_data (os, m.data (), -4, m.numel ());
     }
   else if (cname == "int64")
     {
       int64NDArray m = tc.int64_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), -8, m.numel ());
+      write_mat5_integer_data (os, m.data (), -8, m.numel ());
     }
   else if (cname == "uint8")
     {
       uint8NDArray m = tc.uint8_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), 1, m.numel ());
+      write_mat5_integer_data (os, m.data (), 1, m.numel ());
     }
   else if (cname == "uint16")
     {
       uint16NDArray m = tc.uint16_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), 2, m.numel ());
+      write_mat5_integer_data (os, m.data (), 2, m.numel ());
     }
   else if (cname == "uint32")
     {
       uint32NDArray m = tc.uint32_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), 4, m.numel ());
+      write_mat5_integer_data (os, m.data (), 4, m.numel ());
     }
   else if (cname == "uint64")
     {
       uint64NDArray m = tc.uint64_array_value ();
 
-      write_mat5_integer_data (os, m.fortran_vec (), 8, m.numel ());
+      write_mat5_integer_data (os, m.data (), 8, m.numel ());
     }
   else if (tc.islogical ())
     {
       uint8NDArray m (tc.bool_array_value ());
 
-      write_mat5_integer_data (os, m.fortran_vec (), 1, m.numel ());
+      write_mat5_integer_data (os, m.data (), 1, m.numel ());
     }
   else if (tc.is_real_scalar () || tc.is_real_matrix () || tc.is_range ())
     {
@@ -2601,8 +2706,9 @@ save_mat5_binary_element (std::ostream& os,
     {
       if (tc.is_inline_function () || tc.isobject ())
         {
-          std::string classname = (tc.isobject () ? tc.class_name () : "inline");
-          size_t namelen = classname.length ();
+          std::string classname = (tc.isobject () ? tc.class_name ()
+                                   : "inline");
+          std::size_t namelen = classname.length ();
 
           if (namelen > max_namelen)
             namelen = max_namelen; // Truncate names if necessary
@@ -2618,8 +2724,7 @@ save_mat5_binary_element (std::ostream& os,
 
       octave_map m;
 
-      octave::load_path& lp
-        = octave::__get_load_path__ ("save_mat5_binary_element");
+      octave::load_path& lp = octave::__get_load_path__ ();
 
       if (tc.isobject ()
           && lp.find_method (tc.class_name (), "saveobj") != "")
@@ -2683,9 +2788,9 @@ save_mat5_binary_element (std::ostream& os,
             for (octave_idx_type i = 0; i < nf; i++)
               {
                 bool retval2 = save_mat5_binary_element (os, elts[i][j], "",
-                                                         mark_global,
-                                                         false,
-                                                         save_as_floats);
+                               mark_global,
+                               false,
+                               save_as_floats);
                 if (! retval2)
                   error ("save: error while writing '%s' to MAT file",
                          name.c_str ());
