@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2022 The Octave Project Developers
+// Copyright (C) 1996-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -40,20 +40,29 @@
 #include "mx-base.h"
 #include "str-vec.h"
 
+#include "auto-shlib.h"
 #include "oct-hdf5-types.h"
 #include "oct-stream.h"
 
-namespace octave
-{
-  class stack_frame;
-  class type_info;
+OCTAVE_BEGIN_NAMESPACE(octave)
 
-  // FIXME: This is not ideal, but it avoids including
-  // interpreter-private.h here and bringing in a lot of unnecessary
-  // symbols that require even more header files.
+class stack_frame;
+class type_info;
 
-  extern OCTINTERP_API type_info& __get_type_info__ (const std::string&);
-}
+// FIXME: This is not ideal, but it avoids including
+// interpreter-private.h here and bringing in a lot of unnecessary
+// symbols that require even more header files.  Since the typeinfo
+// object is required to load a user-defined octave_value object,
+// maybe this function should be declared in a public header file?
+
+extern OCTINTERP_API type_info& __get_type_info__ (void);
+
+// For now just preserve the old interface and don't mark it as deprecated.
+// This function is currently an internal, private function.  Additional
+// changes may be made before version 8 is finally released.
+inline type_info& __get_type_info__ (const std::string&) { return __get_type_info__ (); }
+
+OCTAVE_END_NAMESPACE(octave)
 
 class Cell;
 class float_display_format;
@@ -204,8 +213,7 @@ DEF_BTYP_TRAITS (btyp_char, char);
   tspec const std::string t::c_name (c);                                \
   tspec void t::register_type (void)                                    \
   {                                                                     \
-    octave::type_info& type_info                                        \
-      = octave::__get_type_info__ (#t "::register_type");               \
+    octave::type_info& type_info = octave::__get_type_info__ ();        \
                                                                         \
     register_type (type_info);                                          \
   }                                                                     \
@@ -231,7 +239,7 @@ octave_base_value
 {
 public:
 
-  typedef octave_base_value * (*type_conv_fcn) (const octave_base_value&);
+  typedef octave_base_value *(*type_conv_fcn) (const octave_base_value&);
 
   // type conversion, including result type information
   class type_conv_info
@@ -301,7 +309,8 @@ public:
   virtual octave_value as_uint32 (void) const;
   virtual octave_value as_uint64 (void) const;
 
-  virtual octave_base_value * try_narrowing_conversion (void) { return nullptr; }
+  virtual octave_base_value * try_narrowing_conversion (void)
+  { return nullptr; }
 
   virtual void maybe_economize (void) { }
 
@@ -385,6 +394,8 @@ public:
 
   virtual bool is_storable (void) const { return true; }
 
+  virtual bool is_legacy_object (void) const { return false; }
+
   bool isempty (void) const { return (dims ().any_zero ()); }
 
   bool is_zero_by_zero (void) const { return dims().zero_by_zero (); }
@@ -628,10 +639,14 @@ public:
 
   virtual Array<std::string> cellstr_value (void) const;
 
-  virtual octave::range<float> float_range_value (void) const;
-
   virtual octave::range<double> range_value (void) const;
 
+  // For now, disable all but range<double>.
+
+#if 0
+
+  virtual octave::range<float> float_range_value (void) const;
+
   virtual octave::range<octave_int8> int8_range_value (void) const;
 
   virtual octave::range<octave_int16> int16_range_value (void) const;
@@ -648,12 +663,16 @@ public:
 
   virtual octave::range<octave_uint64> uint64_range_value (void) const;
 
+#endif
+
   virtual octave_map map_value (void) const;
 
   virtual octave_scalar_map scalar_map_value (void) const;
 
   virtual string_vector map_keys (void) const;
 
+  virtual bool isfield (const std::string&) const;
+
   virtual std::size_t nparents (void) const;
 
   virtual std::list<std::string> parent_class_name_list (void) const;
@@ -913,8 +932,27 @@ private:
   DECLARE_OV_BASE_TYPEID_FUNCTIONS_AND_DATA
 };
 
-// TRUE means to perform automatic sparse to real mutation if there
-// is memory to be saved
+class
+OCTINTERP_API
+octave_base_dld_value : public octave_base_value
+{
+public:
+
+  octave_base_dld_value (void) = default;
+
+  ~octave_base_dld_value (void)
+  {
+    m_containing_dynamic_library.delete_later ();
+  }
+
+  octave_base_dld_value (const octave_base_dld_value&) = default;
+
+private:
+
+  octave::auto_shlib m_containing_dynamic_library;
+};
+
+OCTAVE_DEPRECATED (8, "Vsparse_auto_mutate is obsolete and is now always false")
 extern OCTINTERP_API bool Vsparse_auto_mutate;
 
 // Utility function to convert C++ arguments used in subsref/subsasgn into an
