@@ -1,24 +1,27 @@
-/*
-
-Copyright (C) 1996-2019 John W. Eaton
-
-This file is part of Octave.
-
-Octave is free software: you can redistribute it and/or modify it
-under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
-
-Octave is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, see
-<https://www.gnu.org/licenses/>.
-
-*/
+////////////////////////////////////////////////////////////////////////
+//
+// Copyright (C) 1996-2023 The Octave Project Developers
+//
+// See the file COPYRIGHT.md in the top-level directory of this
+// distribution or <https://octave.org/copyright/>.
+//
+// This file is part of Octave.
+//
+// Octave is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Octave is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Octave; see the file COPYING.  If not, see
+// <https://www.gnu.org/licenses/>.
+//
+////////////////////////////////////////////////////////////////////////
 
 #if ! defined (octave_op_int_h)
 #define octave_op_int_h 1
@@ -198,11 +201,7 @@ along with Octave; see the file COPYING.  If not, see
     const octave_ ## T1 ## scalar& v1 = dynamic_cast<const octave_ ## T1 ## scalar&> (a1); \
     const octave_ ## T2 ## scalar& v2 = dynamic_cast<const octave_ ## T2 ## scalar&> (a2); \
                                                                         \
-    if (! v2.T2 ## scalar_value ())                                     \
-      warn_divide_by_zero ();                                           \
-                                                                        \
-    octave_value retval = octave_value (v1.T1 ## scalar_value () / v2.T2 ## scalar_value ()); \
-    return retval;                                                      \
+    return octave_value (v1.T1 ## scalar_value () / v2.T2 ## scalar_value ()); \
   }                                                                     \
                                                                         \
   DEFINTBINOP_FN (PFX ## _pow, T1 ## scalar, T2 ## scalar, xpow, T3, ^) \
@@ -212,11 +211,7 @@ along with Octave; see the file COPYING.  If not, see
     const octave_ ## T1 ## scalar& v1 = dynamic_cast<const octave_ ## T1 ## scalar&> (a1); \
     const octave_ ## T2 ## scalar& v2 = dynamic_cast<const octave_ ## T2 ## scalar&> (a2); \
                                                                         \
-    if (! v1.T1 ## scalar_value ())                                     \
-      warn_divide_by_zero ();                                           \
-                                                                        \
-    octave_value retval = octave_value (v2.T2 ## scalar_value () / v1.T1 ## scalar_value ()); \
-    return retval;                                                      \
+    return octave_value (v2.T2 ## scalar_value () / v1.T1 ## scalar_value ()); \
   }                                                                     \
                                                                         \
   DEFINTBINOP_OP (PFX ## _el_mul, T1 ## scalar, T2 ## scalar, *, T3)    \
@@ -226,11 +221,7 @@ along with Octave; see the file COPYING.  If not, see
     const octave_ ## T1 ## scalar& v1 = dynamic_cast<const octave_ ## T1 ## scalar&> (a1); \
     const octave_ ## T2 ## scalar& v2 = dynamic_cast<const octave_ ## T2 ## scalar&> (a2); \
                                                                         \
-    if (! v2.T2 ## scalar_value ())                                     \
-      warn_divide_by_zero ();                                           \
-                                                                        \
-    octave_value retval = octave_value (v1.T1 ## scalar_value () / v2.T2 ## scalar_value ()); \
-    return retval;                                                      \
+    return octave_value (v1.T1 ## scalar_value () / v2.T2 ## scalar_value ()); \
   }                                                                     \
                                                                         \
   DEFINTBINOP_FN (PFX ## _el_pow, T1 ## scalar, T2 ## scalar, xpow, T3, .^) \
@@ -240,11 +231,7 @@ along with Octave; see the file COPYING.  If not, see
     const octave_ ## T1 ## scalar& v1 = dynamic_cast<const octave_ ## T1 ## scalar&> (a1); \
     const octave_ ## T2 ## scalar& v2 = dynamic_cast<const octave_ ## T2 ## scalar&> (a2); \
                                                                         \
-    if (! v1.T1 ## scalar_value ())                                     \
-      warn_divide_by_zero ();                                           \
-                                                                        \
-    octave_value retval = octave_value (v2.T2 ## scalar_value () / v1.T1 ## scalar_value ()); \
-    return retval;                                                      \
+    return octave_value (v2.T2 ## scalar_value () / v1.T1 ## scalar_value ()); \
   }
 
 #define OCTAVE_SS_INT_BOOL_OPS(PFX, T1, T2, Z1, Z2)                     \
@@ -273,31 +260,31 @@ along with Octave; see the file COPYING.  If not, see
   DEFBINOP_OP (PFX ## _ne, T1 ## scalar, T2 ## scalar, !=)
 
 #define OCTAVE_SS_POW_OPS(T1, T2)                       \
-  octave_value                                          \
+  static octave_value                                   \
   xpow (const octave_ ## T1& a, const octave_ ## T2& b) \
   {                                                     \
     return pow (a, b);                                  \
   }                                                     \
                                                         \
-  octave_value                                          \
+  static octave_value                                   \
   xpow (const octave_ ## T1& a, double b)               \
   {                                                     \
     return pow (a, b);                                  \
   }                                                     \
                                                         \
-  octave_value                                          \
+  static octave_value                                   \
   xpow (double a, const octave_ ## T1& b)               \
   {                                                     \
     return pow (a, b);                                  \
   }                                                     \
                                                         \
-  octave_value                                          \
+  static octave_value                                   \
   xpow (const octave_ ## T1& a, float b)                \
   {                                                     \
     return powf (a, b);                                 \
   }                                                     \
                                                         \
-  octave_value                                          \
+  static octave_value                                   \
   xpow (float a, const octave_ ## T1& b)                \
   {                                                     \
     return powf (a, b);                                 \
@@ -347,11 +334,7 @@ along with Octave; see the file COPYING.  If not, see
    const octave_ ## TS ## scalar& v1 = dynamic_cast<const octave_ ## TS ## scalar&> (a1); \
    const octave_ ## TM ## matrix& v2 = dynamic_cast<const octave_ ## TM ## matrix&> (a2); \
                                                                         \
-   if (! v1.TS ## scalar_value ())                                      \
-     warn_divide_by_zero ();                                            \
-                                                                        \
-   octave_value retval = octave_value (v2.TM ## array_value () / v1.TS ## scalar_value ()); \
-   return retval;                                                       \
+   return octave_value (v2.TM ## array_value () / v1.TS ## scalar_value ()); \
    }                                                                    \
                                                                         \
   DEFINTNDBINOP_OP (PFX ## _el_mul, TS ## scalar, TM ## matrix, TS ## scalar, TM ## array, *, TI) \
@@ -371,11 +354,7 @@ along with Octave; see the file COPYING.  If not, see
    const octave_ ## TS ## scalar& v1 = dynamic_cast<const octave_ ## TS ## scalar&> (a1); \
    const octave_ ## TM ## matrix& v2 = dynamic_cast<const octave_ ## TM ## matrix&> (a2); \
                                                                         \
-   if (! v1.TS ## scalar_value ())                                      \
-     warn_divide_by_zero ();                                            \
-                                                                        \
-   octave_value retval = octave_value (v2.TM ## array_value () / v1.TS ## scalar_value ()); \
-   return retval;                                                       \
+   return octave_value (v2.TM ## array_value () / v1.TS ## scalar_value ()); \
    }
 
 #define OCTAVE_SM_INT_CMP_OPS(PFX, TS, TM)                              \
@@ -393,7 +372,7 @@ along with Octave; see the file COPYING.  If not, see
   DEFNDBINOP_FN (PFX ## _el_or_not,  TS ## scalar, TM ## matrix, TS ## scalar, TM ## array, mx_el_or_not)
 
 #define OCTAVE_SM_POW_OPS(T1, T2)                               \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const octave_ ## T1& a, const T2 ## NDArray& b)    \
   {                                                             \
     T2 ## NDArray result (b.dims ());                           \
@@ -405,7 +384,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const octave_ ## T1& a, const NDArray& b)          \
   {                                                             \
     T1 ## NDArray result (b.dims ());                           \
@@ -417,7 +396,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (double a, const T2 ## NDArray& b)                  \
   {                                                             \
     T2 ## NDArray result (b.dims ());                           \
@@ -429,7 +408,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const octave_ ## T1& a, const FloatNDArray& b)     \
   {                                                             \
     T1 ## NDArray result (b.dims ());                           \
@@ -441,7 +420,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (float a, const T2 ## NDArray& b)                   \
   {                                                             \
     T2 ## NDArray result (b.dims ());                           \
@@ -494,11 +473,7 @@ along with Octave; see the file COPYING.  If not, see
     const octave_ ## TM ## matrix& v1 = dynamic_cast<const octave_ ## TM ## matrix&> (a1); \
     const octave_ ## TS ## scalar& v2 = dynamic_cast<const octave_ ## TS ## scalar&> (a2); \
                                                                         \
-    if (! v2.TS ## scalar_value ())                                     \
-      warn_divide_by_zero ();                                           \
-                                                                        \
-    octave_value retval = octave_value (v1.TM ## array_value () / v2.TS ## scalar_value ()); \
-    return retval;                                                      \
+    return octave_value (v1.TM ## array_value () / v2.TS ## scalar_value ()); \
   }                                                                     \
                                                                         \
   /* DEFBINOP_FN (PFX ## _pow, TM ## matrix, TS ## scalar, xpow) */     \
@@ -521,11 +496,7 @@ along with Octave; see the file COPYING.  If not, see
     const octave_ ## TM ## matrix& v1 = dynamic_cast<const octave_ ## TM ## matrix&> (a1); \
     const octave_ ## TS ## scalar& v2 = dynamic_cast<const octave_ ## TS ## scalar&> (a2); \
                                                                         \
-    if (! v2.TS ## scalar_value ())                                     \
-      warn_divide_by_zero ();                                           \
-                                                                        \
-    octave_value retval = octave_value (v1.TM ## array_value () / v2.TS ## scalar_value ()); \
-    return retval;                                                      \
+    return octave_value (v1.TM ## array_value () / v2.TS ## scalar_value ()); \
   }                                                                     \
                                                                         \
   DEFINTNDBINOP_FN (PFX ## _el_pow, TM ## matrix, TS ## scalar, TM ## array, TS ## scalar, elem_xpow, TI, .^) \
@@ -563,7 +534,8 @@ along with Octave; see the file COPYING.  If not, see
   DEFNDASSIGNOP_OP (PFX ## _assign_div, TM ## matrix, TM ## scalar, TM ## scalar, /=)
 
 #define OCTAVE_MS_POW_OPS(T1, T2)                               \
-  octave_value elem_xpow (T1 ## NDArray a, octave_ ## T2 b)     \
+  static octave_value                                           \
+  elem_xpow (T1 ## NDArray a, octave_ ## T2 b)                  \
   {                                                             \
    T1 ## NDArray result (a.dims ());                            \
    for (int i = 0; i < a.numel (); i++)                         \
@@ -574,7 +546,8 @@ along with Octave; see the file COPYING.  If not, see
    return octave_value (result);                                \
   }                                                             \
                                                                 \
-  octave_value elem_xpow (T1 ## NDArray a, double b)            \
+  static octave_value                                           \
+  elem_xpow (T1 ## NDArray a, double b)                         \
   {                                                             \
     T1 ## NDArray result (a.dims ());                           \
     for (int i = 0; i < a.numel (); i++)                        \
@@ -585,7 +558,8 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value elem_xpow (NDArray a, octave_ ## T2 b)           \
+  static octave_value                                           \
+  elem_xpow (NDArray a, octave_ ## T2 b)                        \
   {                                                             \
     T2 ## NDArray result (a.dims ());                           \
     for (int i = 0; i < a.numel (); i++)                        \
@@ -596,7 +570,8 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value elem_xpow (T1 ## NDArray a, float b)             \
+  static octave_value                                           \
+  elem_xpow (T1 ## NDArray a, float b)                          \
   {                                                             \
     T1 ## NDArray result (a.dims ());                           \
     for (int i = 0; i < a.numel (); i++)                        \
@@ -607,7 +582,8 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value elem_xpow (FloatNDArray a, octave_ ## T2 b)      \
+  static octave_value                                           \
+  elem_xpow (FloatNDArray a, octave_ ## T2 b)                   \
   {                                                             \
     T2 ## NDArray result (a.dims ());                           \
     for (int i = 0; i < a.numel (); i++)                        \
@@ -723,7 +699,7 @@ along with Octave; see the file COPYING.  If not, see
   DEFNDASSIGNOP_FNOP (PFX ## _assign_el_div, TM ## matrix, TM ## matrix, TM ## array, quotient_eq)
 
 #define OCTAVE_MM_POW_OPS(T1, T2)                               \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const T1 ## NDArray& a, const T2 ## NDArray& b)    \
   {                                                             \
     dim_vector a_dims = a.dims ();                              \
@@ -744,7 +720,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const T1 ## NDArray& a, const NDArray& b)          \
   {                                                             \
     dim_vector a_dims = a.dims ();                              \
@@ -765,7 +741,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const NDArray& a, const T2 ## NDArray& b)          \
   {                                                             \
     dim_vector a_dims = a.dims ();                              \
@@ -786,7 +762,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const T1 ## NDArray& a, const FloatNDArray& b)     \
   {                                                             \
     dim_vector a_dims = a.dims ();                              \
@@ -807,7 +783,7 @@ along with Octave; see the file COPYING.  If not, see
     return octave_value (result);                               \
   }                                                             \
                                                                 \
-  octave_value                                                  \
+  static octave_value                                           \
   elem_xpow (const FloatNDArray& a, const T2 ## NDArray& b)     \
   {                                                             \
     dim_vector a_dims = a.dims ();                              \
