@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////
 //
-// Copyright (C) 1996-2021 The Octave Project Developers
+// Copyright (C) 1996-2023 The Octave Project Developers
 //
 // See the file COPYRIGHT.md in the top-level directory of this
 // distribution or <https://octave.org/copyright/>.
@@ -35,6 +35,8 @@
 #include "ov-null-mat.h"
 #include "ops.h"
 
+OCTAVE_BEGIN_NAMESPACE(octave)
+
 // string unary ops.
 
 DEFUNOP (transpose, char_matrix_str)
@@ -217,3 +219,5 @@ install_str_str_ops (octave::type_info& ti)
                        null_assign);
 
 }
+
+OCTAVE_END_NAMESPACE(octave)
