@@ -1,4 +1,9 @@
-## Copyright (C) 2004-2018 John W. Eaton
+########################################################################
+##
+## Copyright (C) 2004-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,24 +20,20 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn {} {} ispc ()
+## @deftypefn {} {@var{tf} =} ispc ()
 ## Return true if Octave is running on a Windows system and false otherwise.
 ## @seealso{isunix, ismac}
 ## @end deftypefn
 
-function retval = ispc ()
+function tf = ispc ()
 
-  if (nargin == 0)
-    retval = __octave_config_info__ ("windows");
-  else
-    print_usage ();
-  endif
+  tf = __octave_config_info__ ("windows");
 
 endfunction
 
 
 %!assert (islogical (ispc ()))
-
-%!error ispc (1)
