@@ -1,30 +1,37 @@
-## Copyright (C) 2008-2017 VZLU Prague, a.s.
+########################################################################
+##
+## Copyright (C) 2008-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
 ##
-## Author: Jaroslav Hajek <highegg@gmail.com>
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn  {} {} fzero (@var{fun}, @var{x0})
-## @deftypefnx {} {} fzero (@var{fun}, @var{x0}, @var{options})
+## @deftypefn  {} {@var{x} =} fzero (@var{fcn}, @var{x0})
+## @deftypefnx {} {@var{x} =} fzero (@var{fcn}, @var{x0}, @var{options})
+## @deftypefnx {} {[@var{x}, @var{fval}] =} fzero (@dots{})
+## @deftypefnx {} {[@var{x}, @var{fval}, @var{info}] =} fzero (@dots{})
 ## @deftypefnx {} {[@var{x}, @var{fval}, @var{info}, @var{output}] =} fzero (@dots{})
 ## Find a zero of a univariate function.
 ##
-## @var{fun} is a function handle, inline function, or string containing the
+## @var{fcn} is a function handle, inline function, or string containing the
 ## name of the function to evaluate.
 ##
 ## @var{x0} should be a two-element vector specifying two points which
@@ -33,7 +40,7 @@
 ## following must hold
 ##
 ## @example
-## sign (@var{fun}(@var{x0}(1))) * sign (@var{fun}(@var{x0}(2))) <= 0
+## sign (@var{fcn}(@var{x0}(1))) * sign (@var{fcn}(@var{x0}(2))) <= 0
 ## @end example
 ##
 ## If @var{x0} is a single scalar then several nearby and distant values are
@@ -42,14 +49,30 @@
 ##
 ## @var{options} is a structure specifying additional options.  Currently,
 ## @code{fzero} recognizes these options:
-## @qcode{"FunValCheck"}, @qcode{"OutputFcn"}, @qcode{"TolX"},
-## @qcode{"MaxIter"}, @qcode{"MaxFunEvals"}.
-## For a description of these options, see @ref{XREFoptimset,,optimset}.
+## @qcode{"Display"}, @qcode{"FunValCheck"}, @qcode{"MaxFunEvals"},
+## @qcode{"MaxIter"}, @qcode{"OutputFcn"}, and @qcode{"TolX"}.
 ##
-## On exit, the function returns @var{x}, the approximate zero point and
-## @var{fval}, the function value thereof.
+## @qcode{"MaxFunEvals"} proscribes the maximum number of function evaluations
+## before the search is halted.  The default value is @code{Inf}.
+## The value must be a positive integer.
 ##
-## @var{info} is an exit flag that can have these values:
+## @qcode{"MaxIter"} proscribes the maximum number of algorithm iterations
+## before the search is halted.  The default value is @code{Inf}.
+## The value must be a positive integer.
+##
+## @qcode{"TolX"} specifies the termination tolerance for the solution @var{x}.
+## The default value is @code{eps}.
+##
+## For a description of the other options,
+## @pxref{XREFoptimset,,@code{optimset}}.
+## To initialize an options structure with default values for @code{fzero} use
+## @code{options = optimset ("fzero")}.
+##
+## On exit, the function returns @var{x}, the approximate zero point, and
+## @var{fval}, the function evaluated at @var{x}.
+##
+## The third output @var{info} reports whether the algorithm succeeded and
+## may take one of the following values:
 ##
 ## @itemize
 ## @item 1
@@ -59,7 +82,7 @@
 ##  Maximum number of iterations or function evaluations has been reached.
 ##
 ## @item -1
-## The algorithm has been terminated from user output function.
+## The algorithm has been terminated by a user @code{OutputFcn}.
 ##
 ## @item -5
 ## The algorithm may have converged to a singular point.
@@ -72,9 +95,12 @@
 ## @item iterations
 ##  Number of iterations through loop.
 ##
-## @item nfev
+## @item @nospell{funcCount}
 ##  Number of function evaluations.
 ##
+## @item algorithm
+##  The string @qcode{"bisection, interpolation"}.
+##
 ## @item bracketx
 ##  A two-element vector with the final bracketing of the zero along the
 ## x-axis.
@@ -86,10 +112,10 @@
 ## @seealso{optimset, fsolve}
 ## @end deftypefn
 
-## This is essentially the ACM algorithm 748: Enclosing Zeros of
-## Continuous Functions due to Alefeld, Potra and Shi, ACM Transactions
-## on Mathematical Software, Vol. 21, No. 3, September 1995. Although
-## the workflow should be the same, the structure of the algorithm has
+## This is essentially the @nospell{ACM} algorithm 748: Enclosing Zeros of
+## Continuous Functions due to Alefeld, Potra and Shi, @nospell{ACM}
+## Transactions on Mathematical Software, Vol. 21, No. 3, September 1995.
+## Although the workflow should be the same, the structure of the algorithm has
 ## been transformed non-trivially; instead of the authors' approach of
 ## sequentially calling building blocks subprograms we implement here a
 ## FSM version using one interior point determination and one bracketing
@@ -101,40 +127,50 @@
 ## PKG_ADD: ## Discard result to avoid polluting workspace with ans at startup.
 ## PKG_ADD: [~] = __all_opts__ ("fzero");
 
-function [x, fval, info, output] = fzero (fun, x0, options = struct ())
+function [x, fval, info, output] = fzero (fcn, x0, options = struct ())
 
   ## Get default options if requested.
-  if (nargin == 1 && ischar (fun) && strcmp (fun, 'defaults'))
-    x = optimset ("MaxIter", Inf, "MaxFunEvals", Inf, "TolX", eps,
-                  "OutputFcn", [], "FunValCheck", "off");
+  if (nargin == 1 && ischar (fcn) && strcmp (fcn, "defaults"))
+    x = struct ("Display", "notify", "FunValCheck", "off",
+                "MaxFunEvals", Inf, "MaxIter", Inf,
+                "OutputFcn", [], "TolX", eps);
     return;
   endif
 
-  if (nargin < 2 || nargin > 3)
+  if (nargin < 2)
     print_usage ();
   endif
 
-  if (ischar (fun))
-    fun = str2func (fun, "global");
+  if (ischar (fcn))
+    fcn = str2func (fcn);
   endif
 
-  ## FIXME:
-  ## displev = optimget (options, "Display", "notify");
+  displev = optimget (options, "Display", "notify");
+  switch (displev)
+    case "iter"
+      displev = 1;
+    case "final"
+      displev = 2;
+    case "notify"
+      displev = 3;
+    otherwise  # "none"
+      displev = 0;
+  endswitch
+
   funvalchk = strcmpi (optimget (options, "FunValCheck", "off"), "on");
+  maxfev = optimget (options, "MaxFunEvals", Inf);
+  maxiter = optimget (options, "MaxIter", Inf);
   outfcn = optimget (options, "OutputFcn");
   tolx = optimget (options, "TolX", eps);
-  maxiter = optimget (options, "MaxIter", Inf);
-  maxfev = optimget (options, "MaxFunEvals", Inf);
 
-  persistent mu = 0.5;
+  mu = 0.5;
 
   if (funvalchk)
-    ## Replace fun with a guarded version.
-    fun = @(x) guarded_eval (fun, x);
+    ## Replace fcn with a guarded version.
+    fcn = @(x) guarded_eval (fcn, x);
   endif
 
-  ## The default exit flag if exceeded number of iterations.
-  info = 0;
+  info = 0;  # The default exit flag if number of iterations exceeded.
   niter = 0;
   nfev = 0;
 
@@ -142,21 +178,39 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
 
   ## Prepare...
   a = x0(1);
-  fa = fun (a);
+  fa = fcn (a);
   nfev = 1;
   if (length (x0) > 1)
     b = x0(2);
-    fb = fun (b);
+    fb = fcn (b);
     nfev += 1;
   else
-    ## Try to get b.
-    if (a == 0)
-      aa = 1;
+    ## Try to find a value for b which brackets a zero-crossing
+    if (displev == 1)
+      printf ( ...
+        "\nSearch for an interval around %g containing a sign change:\n", a);
+      printf (" Fcn-count    a          f(a)             b          ");
+      printf ("f(b)        Procedure\n");
+      fmt_str = " %4d  %13.6g %13.6g %13.6g %13.6g   %s\n";
+    endif
+
+    ## For very small values, switch to absolute rather than relative search
+    if (abs (a) < .001)
+      aa = ifelse (a == 0, 0.1, sign (a) * 0.1);
     else
       aa = a;
     endif
-    for b = [0.9*aa, 1.1*aa, aa-1, aa+1, 0.5*aa 1.5*aa, -aa, 2*aa, -10*aa, 10*aa]
-      fb = fun (b); nfev += 1;
+    if (displev == 1)
+      printf (fmt_str, nfev, a, fa, a, fa, "initial interval");
+    endif
+    ## Search in an ever-widening range around the initial point.
+    for srch = [-.01 +.025 -.05 +.10 -.25 +.50 -1 +2.5 -5 +10 -50 +100 -500 +1000]
+      b = aa + aa*srch;
+      fb = fcn (b);
+      nfev += 1;
+      if (displev == 1)
+        printf (fmt_str, nfev, a, fa, b, fb, "search");
+      endif
       if (sign (fa) * sign (fb) <= 0)
         break;
       endif
@@ -174,7 +228,13 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
   endif
 
   if (! (sign (fa) * sign (fb) <= 0))
-    error ("fzero:bracket", "fzero: not a valid initial bracketing");
+    error ("Octave:fzero:bracket", "fzero: not a valid initial bracketing");
+  endif
+
+  if (displev == 1)
+    printf ("\nSearch for a zero in the interval [%g, %g]:\n", a, b);
+    disp (" Fcn-count    x          f(x)             Procedure");
+    fmt_str = " %4d  %13.6g %13.6g        %s\n";
   endif
 
   slope0 = (fb - fa) / (b - a);
@@ -194,6 +254,9 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
   else
     u = b; fu = fb;
   endif
+  if (displev == 1)
+    printf (fmt_str, nfev, u, fu, "initial");
+  endif
 
   if (isa (x0, "single") || isa (fa, "single"))
     macheps = eps ("single");
@@ -238,7 +301,7 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
           c = a + q31 + q32 + q33;
         endif
         if (l < 4 || sign (c - a) * sign (c - b) > 0)
-          ## Quadratic interpolation + newton.
+          ## Quadratic interpolation + Newton.
           a0 = fa;
           a1 = (fb - fa)/(b - a);
           a2 = ((fd - fb)/(d - b) - a1) / (d - a);
@@ -282,8 +345,11 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
 
     ## Calculate new point.
     x = c;
-    fval = fc = fun (c);
+    fval = fc = fcn (c);
     niter += 1; nfev += 1;
+    if (displev == 1)
+      printf (fmt_str, nfev, x, fc, "interpolation");
+    endif
 
     ## Modification 2: skip inverse cubic interpolation if
     ## nonmonotonicity is detected.
@@ -308,7 +374,7 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
       break;
     else
       ## This should never happen.
-      error ("fzero:bracket", "fzero: zero point is not bracketed");
+      error ("Octave:fzero:bracket", "fzero: zero point is not bracketed");
     endif
 
     ## If there's an output function, use it now.
@@ -349,22 +415,41 @@ function [x, fval, info, output] = fzero (fun, x0, options = struct ())
     endif
   endif
 
+  if (displev != 0)
+    switch (info)
+      case 1
+        if (displev != 3)
+          disp ("\nAlgorithm converged.\n");
+        endif
+      case -1
+        disp ("\nAlgorithm has been terminated by user.\n");
+      case -5
+        disp ("\nAlgorithm seemingly converged to a singular point.\n");
+      otherwise
+        disp ( ...
+          "\nMaximum number of iterations or function evaluations reached.\n");
+    endswitch
+  endif
+
   output.iterations = niter;
   output.funcCount = nfev;
+  output.algorithm = "bisection, interpolation";
   output.bracketx = [a, b];
   output.brackety = [fa, fb];
 
 endfunction
 
 ## A helper function that evaluates a function and checks for bad results.
-function fx = guarded_eval (fun, x)
-  fx = fun (x);
+function fx = guarded_eval (fcn, x)
+
+  fx = fcn (x);
   fx = fx(1);
   if (! isreal (fx))
-    error ("fzero:notreal", "fzero: non-real value encountered");
+    error ("Octave:fzero:notreal", "fzero: non-real value encountered");
   elseif (isnan (fx))
-    error ("fzero:isnan", "fzero: NaN value encountered");
+    error ("Octave:fzero:isnan", "fzero: NaN value encountered");
   endif
+
 endfunction
 
 
@@ -372,3 +457,5 @@ endfunction
 %! opt0 = optimset ("tolx", 0);
 %!assert (fzero (@cos, [0, 3], opt0), pi/2, 10*eps)
 %!assert (fzero (@(x) x^(1/3) - 1e-8, [0,1], opt0), 1e-24, 1e-22*eps)
+%!assert <*54445> (fzero (@ (x) x, 0), 0)
+%!assert <*54445> (fzero (@ (x) x + 1, 0), -1)
