@@ -1,4 +1,9 @@
-## Copyright (C) 1996-2018 Kurt Hornik
+########################################################################
+##
+## Copyright (C) 1996-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
@@ -15,13 +20,15 @@
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
 ## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn {} {} deblank (@var{s})
+## @deftypefn {} {@var{s} =} deblank (@var{s})
 ## Remove trailing whitespace and nulls from @var{s}.
 ##
 ## If @var{s} is a matrix, @var{deblank} trims each row to the length of
-## longest string.  If @var{s} is a cell array of strings, operate
+## the longest string.  If @var{s} is a cell array of strings, operate
 ## recursively on each string element.
 ##
 ## Examples:
@@ -38,9 +45,6 @@
 ## @seealso{strtrim}
 ## @end deftypefn
 
-## Author: Kurt Hornik <Kurt.Hornik@wu-wien.ac.at>
-## Adapted-By: jwe
-
 function s = deblank (s)
 
   if (nargin != 1)
@@ -98,5 +102,5 @@ endfunction
 %!assert (deblank ({" abc   ", {"   def   "}}), {" abc", {"   def"}})
 
 %!error <Invalid call to deblank> deblank ()
-%!error <Invalid call to deblank> deblank ("foo", "bar")
+%!error <called with too many inputs> deblank ("foo", "bar")
 %!error <argument must be a string> deblank (1)
